@@ -41,13 +41,15 @@
 #define TRACKER_DB_STATEMENT_SQLITE(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqlite))
 #define TRACKER_DB_STATEMENT_SQLITE_CLASS(c)     (G_TYPE_CHECK_CLASS_CAST ((c),    TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqliteClass))
 #define TRACKER_IS_DB_STATEMENT_SQLITE(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE))
+
 #define TRACKER_IS_DB_STATEMENT_SQLITE_CLASS(c)  (G_TYPE_CHECK_CLASS_TYPE ((o),    TRACKER_TYPE_DB_STATEMENT_SQLITE))
 #define TRACKER_DB_STATEMENT_SQLITE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o),  TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqliteClass))
 
 
 #define TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqlitePrivate))
 
-#define TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqlitePrivate))
+#define TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE(o) (((TrackerDBStatementSqlite *)o)->priv)
+#define TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE_O(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqlitePrivate))
 
 #define TRACKER_DB_CURSOR_SQLITE_GET_PRIVATE_O(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_CURSOR_SQLITE, TrackerDBCursorSqlitePrivate))
 #define TRACKER_DB_CURSOR_SQLITE_GET_PRIVATE(o) (((TrackerDBCursorSqlite *)o)->priv)
@@ -81,6 +83,7 @@ struct TrackerDBInterfaceSqlitePrivate {
 
 	guint in_transaction : 1;
 	guint ro : 1;
+	guint fts_initialized : 1;
 };
 
 struct TrackerDBStatementSqlitePrivate {
@@ -97,6 +100,7 @@ struct TrackerDBCursorSqlitePrivate {
 
 struct TrackerDBStatementSqlite {
 	GObject parent_instance;
+	TrackerDBStatementSqlitePrivate *priv;
 };
 
 struct TrackerDBStatementSqliteClass {
@@ -138,6 +142,7 @@ G_DEFINE_TYPE_WITH_CODE (TrackerDBCursorSqlite, tracker_db_cursor_sqlite, G_TYPE
 void
 tracker_db_interface_sqlite_enable_shared_cache (void)
 {
+	sqlite3_config (SQLITE_CONFIG_MULTITHREAD);
 	sqlite3_enable_shared_cache (1);
 }
 
@@ -218,6 +223,129 @@ function_sparql_string_from_filename (sqlite3_context *context,
 	sqlite3_result_text (context, name, -1, g_free);
 }
 
+static void
+function_sparql_uri_is_parent (sqlite3_context *context,
+                               int              argc,
+                               sqlite3_value   *argv[])
+{
+	const gchar *uri, *parent, *remaining;
+	gboolean match = FALSE;
+	guint parent_len;
+
+	if (argc != 2) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	parent = sqlite3_value_text (argv[0]);
+	uri = sqlite3_value_text (argv[1]);
+
+	if (!parent || !uri) {
+		sqlite3_result_error (context, "Invalid arguments", -1);
+		return;
+	}
+
+	/* Check only one argument, it's going to
+	 * be compared with the other anyway.
+	 */
+	if (strstr (parent, "://") == NULL) {
+		sqlite3_result_int (context, FALSE);
+		return;
+	}
+
+	parent_len = strlen (parent);
+
+	/* Remove trailing '/', will
+	 * be checked later on uri.
+	 */
+	while (parent[parent_len - 1] == '/') {
+		parent_len--;
+	}
+
+	if (strncmp (uri, parent, parent_len) == 0 && uri[parent_len] == '/') {
+		const gchar *slash;
+
+		while (uri[parent_len] == '/') {
+			parent_len++;
+		}
+
+		remaining = &uri[parent_len];
+
+		if (*remaining == '\0') {
+			/* Exact match, not a child */
+			match = FALSE;
+		} else if ((slash = strchr (remaining, '/')) == NULL) {
+			/* Remaining doesn't have uri
+			 * separator, it's a direct child.
+			 */
+			match = TRUE;
+		} else {
+			/* Check it's not trailing slashes */
+			while (*slash == '/') {
+				slash++;
+			}
+
+			match = (*slash == '\0');
+		}
+	}
+
+	sqlite3_result_int (context, match);
+}
+
+static void
+function_sparql_uri_is_descendant (sqlite3_context *context,
+                                   int              argc,
+                                   sqlite3_value   *argv[])
+{
+	const gchar *uri, *parent, *remaining;
+	gboolean match = FALSE;
+	guint parent_len;
+
+	if (argc != 2) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	parent = sqlite3_value_text (argv[0]);
+	uri = sqlite3_value_text (argv[1]);
+
+	if (!parent || !uri) {
+		sqlite3_result_error (context, "Invalid arguments", -1);
+		return;
+	}
+
+	/* Check only one argument, it's going to
+	 * be compared with the other anyway.
+	 */
+	if (strstr (parent, "://") == NULL) {
+		sqlite3_result_int (context, FALSE);
+		return;
+	}
+
+	parent_len = strlen (parent);
+
+	/* Remove trailing '/', will
+	 * be checked later on uri.
+	 */
+	while (parent[parent_len - 1] == '/') {
+		parent_len--;
+	}
+
+	if (strncmp (uri, parent, parent_len) == 0 && uri[parent_len] == '/') {
+		while (uri[parent_len] == '/') {
+			parent_len++;
+		}
+
+		remaining = &uri[parent_len];
+
+		if (remaining && *remaining) {
+			match = TRUE;
+		}
+	}
+
+	sqlite3_result_int (context, match);
+}
+
 static void
 function_sparql_cartesian_distance (sqlite3_context *context,
                                     int              argc,
@@ -396,6 +524,14 @@ open_database (TrackerDBInterfaceSqlitePrivate *priv)
 	                         priv, &function_sparql_string_join,
 	                         NULL, NULL);
 
+	sqlite3_create_function (priv->db, "SparqlUriIsParent", 2, SQLITE_ANY,
+	                         priv, &function_sparql_uri_is_parent,
+	                         NULL, NULL);
+
+	sqlite3_create_function (priv->db, "SparqlUriIsDescendant", 2, SQLITE_ANY,
+	                         priv, &function_sparql_uri_is_descendant,
+	                         NULL, NULL);
+
 	sqlite3_extended_result_codes (priv->db, 0);
 	sqlite3_busy_timeout (priv->db, 100000);
 }
@@ -471,6 +607,12 @@ tracker_db_interface_sqlite_get_property (GObject    *object,
 static void
 close_database (TrackerDBInterfaceSqlitePrivate *priv)
 {
+	gint rc;
+
+	if (priv->fts_initialized) {
+		tracker_fts_shutdown ();
+	}
+
 	g_hash_table_unref (priv->dynamic_statements);
 	priv->dynamic_statements = NULL;
 
@@ -481,7 +623,8 @@ close_database (TrackerDBInterfaceSqlitePrivate *priv)
 	g_slist_free (priv->function_data);
 	priv->function_data = NULL;
 
-	sqlite3_close (priv->db);
+	rc = sqlite3_close (priv->db);
+	g_warn_if_fail (rc == SQLITE_OK);
 }
 
 void
@@ -493,6 +636,7 @@ tracker_db_interface_sqlite_fts_init (TrackerDBInterfaceSqlite *interface,
 	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (interface);
 
 	tracker_fts_init (priv->db, create);
+	priv->fts_initialized = TRUE;
 }
 
 static void
@@ -767,11 +911,23 @@ tracker_db_interface_sqlite_execute_query (TrackerDBInterface  *db_interface,
 	return result_set;
 }
 
+static gboolean
+tracker_db_interface_sqlite_interrupt (TrackerDBInterface *iface)
+{
+	TrackerDBInterfaceSqlitePrivate *priv;
+
+	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (iface);
+	sqlite3_interrupt (priv->db);
+
+	return TRUE;
+}
+
 static void
 tracker_db_interface_sqlite_iface_init (TrackerDBInterfaceIface *iface)
 {
 	iface->create_statement = tracker_db_interface_sqlite_create_statement;
 	iface->execute_query = tracker_db_interface_sqlite_execute_query;
+	iface->interrupt = tracker_db_interface_sqlite_interrupt;
 }
 
 TrackerDBInterface *
@@ -1158,6 +1314,7 @@ tracker_db_cursor_sqlite_iface_init (TrackerDBCursorIface *iface)
 static void
 tracker_db_statement_sqlite_init (TrackerDBStatementSqlite *stmt)
 {
+	stmt->priv = TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE_O(stmt);
 }
 
 static void
