@@ -34,16 +34,32 @@
 #include <time.h>
 #include <inttypes.h>
 
-#include <glib/gi18n-lib.h>
 #include <glib-object.h>
 #include <gio/gio.h>
 
 #include <sqlite3.h>
 
-#include <camel/camel.h>
-#ifndef HAVE_EDS_2_31_2
+#include <camel/camel-mime-message.h>
+#include <camel/camel-i18n.h>
+#include <camel/camel-store.h>
+#include <camel/camel-folder.h>
 #include <camel/camel-db.h>
-#endif
+#include <camel/camel-offline-store.h>
+#include <camel/camel-session.h>
+#include <camel/camel-url.h>
+#include <camel/camel-stream.h>
+#include <camel/camel-stream-mem.h>
+#include <camel/camel-multipart.h>
+#include <camel/camel-multipart-encrypted.h>
+#include <camel/camel-multipart-signed.h>
+#include <camel/camel-medium.h>
+#include <camel/camel-gpg-context.h>
+#include <camel/camel-smime-context.h>
+#include <camel/camel-string-utils.h>
+#include <camel/camel-stream-filter.h>
+#include <camel/camel-stream-null.h>
+#include <camel/camel-mime-filter-charset.h>
+#include <camel/camel-mime-filter-windows.h>
 
 #include <mail/mail-config.h>
 #include <mail/mail-session.h>
@@ -116,8 +132,6 @@ typedef struct {
 	GMutex *mutex;
 	GCond *cond;
 	gboolean has_happened;
-	gpointer pool;
-	gboolean dont_free;
 } PoolItem;
 
 typedef struct {
@@ -287,16 +301,8 @@ get_email_and_fullname (const gchar *line, gchar **email, gchar **fullname)
 static void
 folder_registry_free (FolderRegistry *registry)
 {
-#ifdef HAVE_EDS_2_31_2
-	g_signal_handler_disconnect (registry->folder, registry->hook_info->hook_id);
-#else
 	camel_object_remove_event (registry->folder, registry->hook_info->hook_id);
-#endif
-#ifdef HAVE_EDS_2_31_2
-	g_object_unref (registry->folder);
-#else
 	camel_object_unref (registry->folder);
-#endif
 	g_free (registry->hook_info->account_uri);
 	g_slice_free (OnSummaryChangedInfo, registry->hook_info);
 	g_slice_free (FolderRegistry, registry);
@@ -313,11 +319,7 @@ folder_registry_new (const gchar *account_uri,
 	registry->hook_info->account_uri = g_strdup (account_uri);
 	registry->hook_info->self = self; /* weak */
 	registry->hook_info->hook_id = 0;
-#ifdef HAVE_EDS_2_31_2
-	g_object_ref (folder);
-#else
 	camel_object_ref (folder);
-#endif
 	registry->folder = folder;
 
 	return registry;
@@ -337,7 +339,6 @@ static void
 thread_pool_exec (gpointer data, gpointer user_data)
 {
 	ThreadPool *pool = user_data;
-	PoolItem *item;
 	gboolean dying;
 
 	g_mutex_lock (pool->mutex);
@@ -345,13 +346,10 @@ thread_pool_exec (gpointer data, gpointer user_data)
 	pool->items = g_list_remove (pool->items, data);
 	g_mutex_unlock (pool->mutex);
 
-
 	if (!dying)
 		pool->func (data, pool->cancel);
 
-	item = data;
-	if (!item->dont_free)
-		pool->freeup (data, pool->cancel);
+	pool->freeup (data, pool->cancel);
 }
 
 
@@ -359,7 +357,6 @@ static void
 reply_void (GError *error, gpointer  user_data)
 {
 	PoolItem *item = user_data;
-	ThreadPool *pool = item->pool;
 
 	if (error) {
 		g_debug ("Tracker plugin: Error updating data: %s\n", error->message);
@@ -369,9 +366,6 @@ reply_void (GError *error, gpointer  user_data)
 	g_cond_broadcast (item->cond);
 	item->has_happened = TRUE;
 	g_mutex_unlock (item->mutex);
-
-	if (item->dont_free)
-		pool->freeup (item, pool->cancel);
 }
 
 static void
@@ -401,7 +395,6 @@ exec_update (gpointer data, gpointer user_data)
 		g_get_current_time (&val);
 		g_time_val_add (&val, 5 * 1000000); /* 5 seconds worth of patience */
 		no_patience = g_cond_timed_wait (item->cond, item->mutex, &val);
-		item->dont_free = !no_patience;
 	}
 	g_mutex_unlock (item->mutex);
 
@@ -444,11 +437,9 @@ thread_pool_new (GFunc func, GFunc freeup, GCompareDataFunc sorter)
 static void
 thread_pool_push (ThreadPool *pool, gpointer item, gpointer user_data)
 {
-	g_mutex_lock (pool->mutex);
 	pool->items = g_list_prepend (pool->items, item);
 	if (!pool->dying)
 		g_thread_pool_push (pool->pool, item, user_data);
-	g_mutex_unlock (pool->mutex);
 }
 
 static gpointer
@@ -489,8 +480,6 @@ send_sparql_update (TrackerEvolutionPlugin *self, const gchar *sparql, gint prio
 		if (!sparql_pool)
 			sparql_pool = thread_pool_new (exec_update, free_pool_item, pool_sort_func);
 
-		item->pool = sparql_pool;
-		item->dont_free = FALSE;
 		item->prio = prio;
 		item->commit = FALSE;
 		item->client = g_object_ref (priv->client);
@@ -524,8 +513,6 @@ send_sparql_commit (TrackerEvolutionPlugin *self, gboolean update)
 		if (!sparql_pool)
 			sparql_pool = thread_pool_new (exec_update, free_pool_item, pool_sort_func);
 
-		item->pool = sparql_pool;
-		item->dont_free = FALSE;
 		item->prio = 0;
 		item->commit = TRUE;
 		item->client = g_object_ref (priv->client);
@@ -760,7 +747,6 @@ on_folder_summary_changed (CamelFolder *folder,
 
 	summary = folder->summary;
 	em_uri = em_uri_from_camel (account_uri);
-	em_uri [strlen (em_uri) - 1] = '\0';
 
 	merged = g_ptr_array_new ();
 
@@ -827,10 +813,10 @@ on_folder_summary_changed (CamelFolder *folder,
 			/* This is not a path but a URI, don't use the
 			 * OS's directory separator here */
 
-			uri = tracker_uri_printf_escaped ("%s/%s/%s",
-			                                  em_uri,
-			                                  camel_folder_get_full_name (folder),
-			                                  uid);
+			uri = g_strdup_printf ("%s%s/%s",
+			                       em_uri,
+			                       camel_folder_get_full_name (folder),
+			                       uid);
 
 			sparql = tracker_sparql_builder_new_update ();
 
@@ -908,17 +894,17 @@ on_folder_summary_changed (CamelFolder *folder,
 		GString *sparql = g_string_new ("");
 
 		for (i = 0; i< changes->uid_removed->len; i++) {
-			gchar *uri;
 
 			/* This is not a path but a URI, don't use the OS's
 			 * directory separator here */
-			uri = tracker_uri_printf_escaped ("%s/%s/%s",
-			                                  em_uri,
-			                                  camel_folder_get_full_name (folder),
-			                                  (char*) changes->uid_removed->pdata[i]);
 
-			g_string_append_printf (sparql, "DELETE FROM <%s> { <%s> a rdfs:Resource }\n ", uri, uri);
-			g_free (uri);
+			g_string_append_printf (sparql, "DELETE FROM <%s%s/%s> { <%s%s/%s> a rdfs:Resource }\n ",
+			                        em_uri,
+			                        camel_folder_get_full_name (folder),
+			                        (char*) changes->uid_removed->pdata[i],
+			                        em_uri,
+			                        camel_folder_get_full_name (folder),
+			                        (char*) changes->uid_removed->pdata[i]);
 		}
 
 		send_sparql_update (info->self, sparql->str, 100);
@@ -951,7 +937,6 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 	}
 
 	em_uri = em_uri_from_camel (account_uri);
-	em_uri [strlen (em_uri) - 1] = '\0';
 
 	while (iter) {
 		guint count = 0;
@@ -1032,7 +1017,8 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 
 				folder = g_hash_table_lookup (priv->cached_folders, iter->full_name);
 
-				uri = tracker_uri_printf_escaped ("%s/%s/%s", em_uri, iter->full_name, uid);
+				uri =  g_strdup_printf ("%s%s/%s", em_uri,
+				                        iter->full_name, uid);
 
 				if (!sparql) {
 					sparql = tracker_sparql_builder_new_update ();
@@ -1155,10 +1141,7 @@ introduce_store_deal_with_deleted (TrackerEvolutionPlugin *self,
 	sqlite3_stmt *stmt = NULL;
 	CamelDB *cdb_r;
 	guint i, ret;
-	gchar *em_uri;
-
-	em_uri = em_uri_from_camel (account_uri);
-	em_uri [strlen (em_uri) - 1] = '\0';
+	gchar *em_uri = em_uri_from_camel (account_uri);
 
 	query = sqlite3_mprintf ("SELECT uid, mailbox "
 	                         "FROM Deletes "
@@ -1200,9 +1183,8 @@ introduce_store_deal_with_deleted (TrackerEvolutionPlugin *self,
 			/* This is not a path but a URI, don't use the OS's
 			 * directory separator here */
 
-			g_ptr_array_add (subjects_a,
-			                 tracker_uri_printf_escaped ("%s/%s/%s", em_uri,
-			                                             mailbox, uid));
+			g_ptr_array_add (subjects_a, g_strdup_printf ("%s%s/%s", em_uri,
+			                                              mailbox, uid));
 
 			if (count > 100) {
 				more = TRUE;
@@ -1263,6 +1245,7 @@ get_last_deleted_time (TrackerEvolutionPlugin *self)
 			EAccount *account = (EAccount *) e_iterator_get (it);
 			CamelProvider *provider;
 			CamelStore *store;
+			CamelException ex;
 			char *uri;
 			CamelDB *cdb_r;
 			sqlite3_stmt *stmt = NULL;
@@ -1270,10 +1253,13 @@ get_last_deleted_time (TrackerEvolutionPlugin *self)
 			guint ret = SQLITE_OK;
 			guint64 latest = smallest;
 
+			camel_exception_init (&ex);
+
 			if (!account->enabled || !(uri = account->source->url))
 				continue;
 
 			if (!(provider = camel_provider_get(uri, NULL))) {
+				camel_exception_clear (&ex);
 				continue;
 			}
 
@@ -1281,7 +1267,8 @@ get_last_deleted_time (TrackerEvolutionPlugin *self)
 				continue;
 			}
 
-			if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
+			if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
+				camel_exception_clear (&ex);
 				continue;
 			}
 
@@ -1336,15 +1323,9 @@ register_on_get_folder (gchar *uri, CamelFolder *folder, gpointer user_data)
 		goto not_ready;
 	}
 
-#ifdef HAVE_EDS_2_31_2
-	hook_id = g_signal_connect (folder, "changed",
-				    G_CALLBACK (on_folder_summary_changed),
-				    registry->hook_info);
-#else
 	hook_id = camel_object_hook_event (folder, "folder_changed",
 	                                   CAMEL_CALLBACK (on_folder_summary_changed),
 	                                   registry->hook_info);
-#endif
 	registry->hook_info->hook_id = hook_id;
 
 	g_hash_table_replace (priv->registered_folders,
@@ -1521,11 +1502,7 @@ free_worker_thread_info (gpointer data, gpointer user_data)
 	/* Ownership was transfered to us in try_again */
 	free_introduction_info (winfo->intro_info);
 	camel_db_close (winfo->cdb_r);
-#ifdef HAVE_EDS_2_31_2
-	g_object_unref (winfo->store);
-#else
 	camel_object_unref (winfo->store);
-#endif
 	camel_folder_info_free (winfo->iter);
 	g_free (winfo);
 }
@@ -1580,11 +1557,7 @@ on_got_folderinfo_introduce (CamelStore *store,
 
 	/* Ownership of these is transfered in try_again */
 
-#ifdef HAVE_EDS_2_31_2
-	g_object_ref (store);
-#else
 	camel_object_ref (store);
-#endif
 	info->store = store;
 	/* This apparently creates a thread */
 	info->cdb_r = camel_db_clone (store->cdb_r, NULL);
@@ -1627,20 +1600,24 @@ introduce_account_to (TrackerEvolutionPlugin *self,
 {
 	CamelProvider *provider;
 	CamelStore *store;
+	CamelException ex;
 	char *uri, *account_uri, *ptr;
 	IntroductionInfo *intro_info;
 
 	if (!account->enabled || !(uri = account->source->url))
 		return;
 
-	if (!(provider = camel_provider_get(uri, NULL))) {
+	camel_exception_init (&ex);
+	if (!(provider = camel_provider_get(uri, &ex))) {
+		camel_exception_clear (&ex);
 		return;
 	}
 
 	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
 		return;
 
-	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
+		camel_exception_clear (&ex);
 		return;
 	}
 
@@ -1661,11 +1638,7 @@ introduce_account_to (TrackerEvolutionPlugin *self,
 
 	mail_get_folderinfo (store, NULL, on_got_folderinfo_introduce, intro_info);
 
-#ifdef HAVE_EDS_2_31_2
-	g_object_unref (store);
-#else
 	camel_object_unref (store);
-#endif
 
 }
 
@@ -1826,11 +1799,7 @@ on_folder_deleted (CamelStore *store,
 
 static void
 on_folder_renamed (CamelStore *store,
-#ifdef HAVE_EDS_2_31_2
-		   gchar *old_name,
-#else
                    CamelRenameInfo *info,
-#endif
                    StoreRegistry *registry)
 {
 	unregister_account (registry->self, registry->account);
@@ -1848,11 +1817,7 @@ store_registry_new (gpointer co,
 	registry->store = co;
 	registry->account = account; /* weak */
 	registry->self = self; /* weak */
-#ifdef HAVE_EDS_2_31_2
-	g_object_ref (co);
-#else
 	camel_object_ref (co);
-#endif
 
 	return registry;
 }
@@ -1860,16 +1825,8 @@ store_registry_new (gpointer co,
 static void
 store_registry_free (StoreRegistry *registry)
 {
-#ifdef HAVE_EDS_2_31_2
-	g_signal_handler_disconnect (registry->store, registry->hook_id);
-#else
 	camel_object_remove_event (registry->store, registry->hook_id);
-#endif
-#ifdef HAVE_EDS_2_31_2
-	g_object_unref (registry->store);
-#else
 	camel_object_unref (registry->store);
-#endif
 	g_slice_free (StoreRegistry, registry);
 }
 
@@ -1897,45 +1854,27 @@ on_got_folderinfo_register (CamelStore *store,
 
 	/* Hook up catching folder changes in the store */
 	registry = store_registry_new (store, account, self);
-#ifdef HAVE_EDS_2_31_2
-	hook_id = g_signal_connect (store, "folder-created",
-				    G_CALLBACK (on_folder_created),
-				    registry);
-#else
 	hook_id = camel_object_hook_event (store, "folder_created",
 	                                   CAMEL_CALLBACK (on_folder_created),
 	                                   registry);
-#endif
 	registry->hook_id = hook_id;
 	g_hash_table_replace (priv->registered_stores,
 	                      GINT_TO_POINTER (hook_id),
 	                      registry);
 
 	registry = store_registry_new (store, account, self);
-#ifdef HAVE_EDS_2_31_2
-	hook_id = g_signal_connect (store, "folder-renamed",
-				    G_CALLBACK (on_folder_renamed),
-				    registry);
-#else
 	hook_id = camel_object_hook_event (store, "folder_renamed",
 	                                   CAMEL_CALLBACK (on_folder_renamed),
 	                                   registry);
-#endif
 	registry->hook_id = hook_id;
 	g_hash_table_replace (priv->registered_stores,
 	                      GINT_TO_POINTER (hook_id),
 	                      registry);
 
 	registry = store_registry_new (store, account, self);
-#ifdef HAVE_EDS_2_31_2
-	hook_id = g_signal_connect (store, "folder-deleted",
-				    G_CALLBACK (on_folder_deleted),
-				    registry);
-#else
 	hook_id = camel_object_hook_event (store, "folder_deleted",
 	                                   CAMEL_CALLBACK (on_folder_deleted),
 	                                   registry);
-#endif
 	registry->hook_id = hook_id;
 	g_hash_table_replace (priv->registered_stores,
 	                      GINT_TO_POINTER (hook_id),
@@ -1960,20 +1899,24 @@ register_account (TrackerEvolutionPlugin *self,
 {
 	CamelProvider *provider;
 	CamelStore *store;
+	CamelException ex;
 	char *uri;
 	RegisterInfo *reg_info;
 
 	if (!account->enabled || !(uri = account->source->url))
 		return;
 
-	if (!(provider = camel_provider_get(uri, NULL))) {
+	camel_exception_init (&ex);
+	if (!(provider = camel_provider_get(uri, &ex))) {
+		camel_exception_clear (&ex);
 		return;
 	}
 
 	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
 		return;
 
-	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
+		camel_exception_clear (&ex);
 		return;
 	}
 
@@ -1988,11 +1931,7 @@ register_account (TrackerEvolutionPlugin *self,
 	/* Get the account's folder-info and register it asynchronously */
 	mail_get_folderinfo (store, NULL, on_got_folderinfo_register, reg_info);
 
-#ifdef HAVE_EDS_2_31_2
-	g_object_unref (store);
-#else
 	camel_object_unref (store);
-#endif
 }
 
 static gboolean
@@ -2031,18 +1970,22 @@ unregister_account (TrackerEvolutionPlugin *self,
 {
 	CamelProvider *provider;
 	CamelStore *store;
+	CamelException ex;
 	char *uri = account->source->url;
 	RegisterInfo *reg_info;
 
 
-	if (!(provider = camel_provider_get(uri, NULL))) {
+	camel_exception_init (&ex);
+	if (!(provider = camel_provider_get(uri, &ex))) {
+		camel_exception_clear (&ex);
 		return;
 	}
 
 	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
 		return;
 
-	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, NULL))) {
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
+		camel_exception_clear (&ex);
 		return;
 	}
 
@@ -2055,11 +1998,7 @@ unregister_account (TrackerEvolutionPlugin *self,
 	/* Get the account's folder-info and unregister asynchronously */
 	mail_get_folderinfo (store, NULL, on_got_folderinfo_unregister, reg_info);
 
-#ifdef HAVE_EDS_2_31_2
-	g_object_unref (store);
-#else
 	camel_object_unref (store);
-#endif
 }
 
 static void
