@@ -28,8 +28,8 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include <libtracker-client/tracker-client.h>
-#include <libtracker-common/tracker-common.h>
+#include <libtracker-client/tracker.h>
+#include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-miner/tracker-miner.h>
 
 #include "tracker-status-client.h"
@@ -135,7 +135,6 @@ signal_handler (int signo)
 		in_loop = TRUE;
 		g_main_loop_quit (main_loop);
 
-		/* Fall through */
 	default:
 		if (g_strsignal (signo)) {
 			g_print ("\n");
